/* 
 * Parser for the program's run control file 
 * Initially meant to just hold a sequential list of channels and has grown since.
 * This should have been done in lex and yacc, looking back.
 * 2002-2003 HShivanan
 * 
 * For licensing terms, see the file COPYING
 * For contact information, see the file AUTHORS
 *
 */
#include <stdlib.h>
#include <string.h>
#include "configscan.h"
#include "stuff.h"
#include "configparser.h"
#include "item.h"

#define PARSE_ERR_MSG "Detected an error while parsing the run-control file\n"

#define next() config_nexttoken(&sc)


#define copy_str(u) u=copy_quoted(scanner->token)

/*Score Syntax:
  score=(points,field,pattern),(points,field,patter)...
  */
#define match(x) if (scanner->tokenid!=x){\
			error(PARSE_ERR_MSG);\
	                  return;\
			}\
		config_nexttoken(scanner)	

#define assert_token(x) if (scanner->tokenid!=x){\
			error (PARSE_ERR_MSG);\
				return;\
			}
char * copy_quoted(char * string)
{
	size_t len;
	char *new;
	len = strlen(string);
	ASSERT(len > 1);
	ASSERT(*string == '"');
	if (len == 2)
		return NULL; /* empty string */
	new = (char *)fn_malloc(len-1); /*because we don't want the quotes so we */
					/*reduce the length by 2 */
	new[len-2]=0;
	memcpy(new,string+1,len-2);
	return new;
}

/* Parse the score field.
 * Syntax:
 * score = (score,category-to-search,pattern),
 *         (score,etc...)
 */
void parse_score(CONFIG_SCANNER *scanner,FN_OPTIONS *fn)
{
	int score;
	int cat;
	config_nexttoken(scanner);
	match(EQUAL);
	while (1)
	{
		match(PARENOPEN);
		score=atoi(scanner->token);
		config_nexttoken(scanner);
		match(COMMA);
	
		/*Nasty, isn't it?*/
		cat=scanner->tokenid;
		cat=(cat==SCORE_TITLE)?CAT_TITLE:(cat==SCORE_DESCRIPTION)?CAT_DESCRIPTION:
			(cat==SCORE_CHANNEL)?CAT_CHANNEL:(cat==SCORE_LINK)?CAT_LINK:CAT_ANY;
	
		config_nexttoken(scanner);
		match(COMMA);
		add_score(&(fn->score_list),score,cat,scanner->token+1,scanner->count-2);
		match(STRING);
		match(PARENCLOSE);
		if (scanner->tokenid!=COMMA)break;
		match(COMMA);
	}
}

void parse_channels(CONFIG_SCANNER *scanner,FN_OPTIONS *fn)
{
	char *u,*c,*t;
	u=c=t=NULL;
	config_nexttoken(scanner);
	match(EQUAL);
	if (scanner->tokenid==COMMENT)
		config_nexttoken(scanner);
	if (scanner->tokenid != PARENOPEN)
	{
		error(PARSE_ERR_MSG);
		return ;
	}
	while(scanner->tokenid==PARENOPEN)
	{
		match(PARENOPEN);
		assert_token(STRING);
		copy_str(u);
		match(STRING);
		if (scanner->tokenid==COMMA)
		{
			match(COMMA);
			assert_token(STRING);
			copy_str(c);
			match(STRING);
		}
		if (scanner->tokenid==COMMA)
		{
			match(COMMA);
			assert_token(STRING);
			copy_str(t);
			match(STRING);
		}
		match(PARENCLOSE);
		add_channel(u,c,t);
		u=NULL;
		c=NULL;
		t=NULL;
		if (scanner->tokenid!=COMMA)break;
		match(COMMA);
	}
	
}
#define fill_field(val,field); if (!strcmp(val,sc.token)){\
					next();\
					if (sc.tokenid!=EQUAL)return BAD_CONFIG_FILE;\
					next();	if (sc.tokenid!=STRING)return BAD_CONFIG_FILE;\
					fn->field=(char *)fn_malloc(sc.count-1);\
					memcpy(fn->field,sc.token+1,sc.count-2);\
					fn->field[sc.count-2]=0;\
					next();\
					break;}

#define flag_field(val,field); if (!strcmp(val,sc.token)){\
									fn->options |= field;\
									next();\
									break;\
								}
					
int parse_config_file(char *file,FN_OPTIONS *fn)
{
	CONFIG_SCANNER sc;
	char *buffer;
	buffer=read_file(file);
	if (buffer==NULL)
	{
		return NO_CONFIG_FILE;
	}
	sc.code=buffer;
	sc.token=(char *)fn_malloc(20);
	sc.capacity=20;
	sc.tokenid=ERROR;
	next();
	while (sc.tokenid!=_NULL)
	{
// 		next();
		switch (sc.tokenid)
		{
			case COMMENT:next();break;
			case SET:next();
					 flag_field("no-proxy",         OPT_NO_PROXY);
#ifdef DEBUG
					 flag_field("show-alloc-stats", OPT_SHOW_ALLOC_STATS);
#endif
					 flag_field("save-xml",         OPT_SAVE_ALL);
					 break;
			case DEFAULT:
						 fill_field("template",template_file);
						 fill_field("report",report_file);
						 fill_field("proxy",proxy);
						 fill_field("mail-to",mail_info->mail_to);
						 fill_field("mail-from",mail_info->mail_from);
						 fill_field("mail-subject",mail_info->mail_subject);
						 fill_field("xml-directory",xml_dir);
						 if (!strcmp("channels",sc.token))
						 {
							 parse_channels(&sc,fn);
							 break;
						 }
						 if (!strcmp("score",sc.token))
						 {
							 parse_score(&sc,fn);
							 break;
						 }
						 
			/* let anything else from "case DEFAULT:" fall through to "default:"
			 * which will handle errors. Thats why we don't have a "break;" here
			 */
			default:
						 error(PARSE_ERR_MSG);
						 error("Unidentified token:%s\n",sc.token);
						 return BAD_CONFIG_FILE;
/*			default:printf("WHAT!!!!%s\n",sc.token);next();*/
		}
	}
	fn_free(sc.token);
	return 1;
}
